v7-0001-add-servicefile-connection-option-feature.patch
application/octet-stream
Filename: v7-0001-add-servicefile-connection-option-feature.patch
Type: application/octet-stream
Part: 0
Patch
Same data as JSON:
GET /api/v1/attachments/:id/patch
the parsed metadata as JSON — format, series position, per-file stats; never the diff bytes.
API reference →
Format: format-patch
Series: patch v7-0001
Subject: add servicefile connection option feature
| File | + | − |
|---|---|---|
| doc/src/sgml/libpq.sgml | 15 | 1 |
| src/interfaces/libpq/fe-connect.c | 24 | 3 |
| src/interfaces/libpq/t/006_service.pl | 64 | 1 |
From 29faf3149035ab4766c7aa4a51f1c77b71bb5aa2 Mon Sep 17 00:00:00 2001
From: Ryo Kanbayashi <ryo.contact@gmail.com>
Date: Sun, 13 Apr 2025 18:24:36 +0900
Subject: [PATCH v7] add servicefile connection option feature
---
doc/src/sgml/libpq.sgml | 16 ++++++-
src/interfaces/libpq/fe-connect.c | 27 +++++++++--
src/interfaces/libpq/t/006_service.pl | 65 ++++++++++++++++++++++++++-
3 files changed, 103 insertions(+), 5 deletions(-)
diff --git a/doc/src/sgml/libpq.sgml b/doc/src/sgml/libpq.sgml
index 37102c235b..cf6c0320e6 100644
--- a/doc/src/sgml/libpq.sgml
+++ b/doc/src/sgml/libpq.sgml
@@ -2320,6 +2320,19 @@ postgresql://%2Fvar%2Flib%2Fpostgresql/dbname
</listitem>
</varlistentry>
+ <varlistentry id="libpq-connect-servicefile" xreflabel="servicefile">
+ <term><literal>servicefile</literal></term>
+ <listitem>
+ <para>
+ This option specifies the name of the per-user connection service file
+ (see <xref linkend="libpq-pgservice"/>).
+ Defaults to <filename>~/.pg_service.conf</filename>, or
+ <filename>%APPDATA%\postgresql\.pg_service.conf</filename> on
+ Microsoft Windows.
+ </para>
+ </listitem>
+ </varlistentry>
+
<varlistentry id="libpq-connect-target-session-attrs" xreflabel="target_session_attrs">
<term><literal>target_session_attrs</literal></term>
<listitem>
@@ -9596,7 +9609,8 @@ myEventProc(PGEventId evtId, void *evtInfo, void *passThrough)
On Microsoft Windows, it is named
<filename>%APPDATA%\postgresql\.pg_service.conf</filename> (where
<filename>%APPDATA%</filename> refers to the Application Data subdirectory
- in the user's profile). A different file name can be specified by
+ in the user's profile). A different file name can be specified using the
+ <literal>servicefile</literal> key word in a libpq connection string or by
setting the environment variable <envar>PGSERVICEFILE</envar>.
The system-wide file is named <filename>pg_service.conf</filename>.
By default it is sought in the <filename>etc</filename> directory
diff --git a/src/interfaces/libpq/fe-connect.c b/src/interfaces/libpq/fe-connect.c
index 0258d9ace3..b6350cb62f 100644
--- a/src/interfaces/libpq/fe-connect.c
+++ b/src/interfaces/libpq/fe-connect.c
@@ -195,6 +195,9 @@ static const internalPQconninfoOption PQconninfoOptions[] = {
"Database-Service", "", 20,
offsetof(struct pg_conn, pgservice)},
+ {"servicefile", "PGSERVICEFILE", NULL, NULL,
+ "Database-Service-File", "", 64, -1},
+
{"user", "PGUSER", NULL, NULL,
"Database-User", "", 20,
offsetof(struct pg_conn, pguser)},
@@ -5904,6 +5907,7 @@ static int
parseServiceInfo(PQconninfoOption *options, PQExpBuffer errorMessage)
{
const char *service = conninfo_getval(options, "service");
+ const char *service_fname = conninfo_getval(options, "servicefile");
char serviceFile[MAXPGPATH];
char *env;
bool group_found = false;
@@ -5923,11 +5927,18 @@ parseServiceInfo(PQconninfoOption *options, PQExpBuffer errorMessage)
return 0;
/*
- * Try PGSERVICEFILE if specified, else try ~/.pg_service.conf (if that
- * exists).
+ * First, check servicefile option on connection string. Second, check
+ * PGSERVICEFILE environment variable. Finally, check ~/.pg_service.conf
+ * (if that exists).
*/
- if ((env = getenv("PGSERVICEFILE")) != NULL)
+ if (service_fname != NULL)
+ {
+ strlcpy(serviceFile, service_fname, sizeof(serviceFile));
+ }
+ else if ((env = getenv("PGSERVICEFILE")) != NULL)
+ {
strlcpy(serviceFile, env, sizeof(serviceFile));
+ }
else
{
char homedir[MAXPGPATH];
@@ -6089,6 +6100,16 @@ parseServiceFile(const char *serviceFile,
goto exit;
}
+ if (strcmp(key, "servicefile") == 0)
+ {
+ libpq_append_error(errorMessage,
+ "nested servicefile specifications not supported in service file \"%s\", line %d",
+ serviceFile,
+ linenr);
+ result = 3;
+ goto exit;
+ }
+
/*
* Set the parameter --- but don't override any previous
* explicit setting.
diff --git a/src/interfaces/libpq/t/006_service.pl b/src/interfaces/libpq/t/006_service.pl
index 4fe5adc5c2..a5c02d9895 100644
--- a/src/interfaces/libpq/t/006_service.pl
+++ b/src/interfaces/libpq/t/006_service.pl
@@ -7,7 +7,7 @@ use PostgreSQL::Test::Cluster;
use Test::More;
# This tests scenarios related to the service name and the service file,
# for the connection options and their environment variables.
my $node = PostgreSQL::Test::Cluster->new('node');
$node->init;
@@ -146,6 +146,69 @@ local $ENV{PGSERVICEFILE} = "$srvfile_empty";
unlink($srvfile_default);
}
+# Backslashes escaped path string for getting collect result at concatenation
+# for Windows environment
+my $srvfile_win_cared = $srvfile_valid;
+$srvfile_win_cared =~ s/\\/\\\\/g;
+
+# Check that servicefile option works as expected
+{
+ $dummy_node->connect_ok(
+ q{service=my_srv servicefile='} . $srvfile_win_cared . q{'},
+ 'service=my_srv servicefile=...',
+ sql => "SELECT 'connect3'",
+ expected_stdout => qr/connect3/
+ );
+
+ # Encode slashes and backslash
+ my $encoded_srvfile = $srvfile_valid =~ s{([\\/])}{
+ $1 eq '/' ? '%2F' : '%5C'
+ }ger;
+
+ # Additionaly encode a colon in servicefile path of Windows
+ $encoded_srvfile =~ s/:/%3A/g;
+
+ $dummy_node->connect_ok(
+ 'postgresql:///?service=my_srv&servicefile=' . $encoded_srvfile,
+ 'postgresql:///?service=my_srv&servicefile=...',
+ sql => "SELECT 'connect4'",
+ expected_stdout => qr/connect4/
+ );
+
+ local $ENV{PGSERVICE} = 'my_srv';
+ $dummy_node->connect_ok(
+ q{servicefile='} . $srvfile_win_cared . q{'},
+ 'envvar: PGSERVICE=my_srv + servicefile=...',
+ sql => "SELECT 'connect5'",
+ expected_stdout => qr/connect5/
+ );
+
+ $dummy_node->connect_ok(
+ 'postgresql://?servicefile=' . $encoded_srvfile,
+ 'envvar: PGSERVICE=my_srv + postgresql://?servicefile=...',
+ sql => "SELECT 'connect6'",
+ expected_stdout => qr/connect6/
+ );
+}
+
+# Check that servicefile option takes precedence over PGSERVICEFILE environment variable
+{
+ local $ENV{PGSERVICEFILE} = 'non-existent-file.conf';
+
+ $dummy_node->connect_fails(
+ 'service=my_srv',
+ 'service=... fails with wrong PGSERVICEFILE',
+ expected_stderr => qr/service file "non-existent-file\.conf" not found/
+ );
+
+ $dummy_node->connect_ok(
+ q{service=my_srv servicefile='} . $srvfile_win_cared . q{'},
+ 'servicefile= takes precedence over PGSERVICEFILE',
+ sql => "SELECT 'connect7'",
+ expected_stdout => qr/connect7/
+ );
+}
+
$node->teardown_node;
done_testing();
@@ -7,7 +7,7 @@ use PostgreSQL::Test::Cluster;
use Test::More;
# This tests scenarios related to the service name and the service file,
-# for the connection options and their environment variables.
+# for the connection options, servicefile options and their environment variables.
my $node = PostgreSQL::Test::Cluster->new('node');
$node->init;
--
2.34.1